/*
 * Copyright 2001-2002, 2010 pilot <pilot@monkey.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>

#include "arirang.h"
#include "version.h"

void
timeout(int sig)
{
    int sock;
#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
    if (sig == SIGALRM)
        sock = SIGALRM;
#else
    close(sock);
#endif
}

/*
 * int sigchld(int sig) { 
    int status; 
    wait(&status); 
    signal(SIGCHLD, sigchld);
 *  }
 */

char *html = NULL;
FILE *htmlreport;
int wflags = 0;
int ftype = 0;
short int CONNECT_TIMEOUT = 5;

int
main(int argc, char *argv[])
{
    int pcount = 0;

    unsigned short int MAX_PROCESSES = 30;
    unsigned short int RULE_PROCESSES = 0;
    unsigned short int IPRANGE_PROCESSES = 30;

    unsigned int http_port = 80;
    unsigned long start = 0, end = 0, count;
    unsigned short int statuscode = 1;

    int ch;
    char *server = NULL;
    char *filename = NULL;
    FILE *scanlist, *tmp_iprange;
    char list[50];

    char *rulefile = NULL;
    int gflags = 0, oflags = 0, hflags = 0, fflags = 0,
        rflags = 0, mflags = 0;

    char *slash = NULL;
    unsigned long cidr =0, mask=0;
    int bits = 0;

    unsigned long fcount = 0;
    char *fip;
    int fnum = 0;
    
    unsigned long sec;
    unsigned long usec;
    struct timeval tv1, tv2;
    gettimeofday(&tv1, 0);

    if (argc < 3)
        usage();

    scrview();
    opterr = 0;

    while ((ch = getopt(argc, argv, "t:w:GOp:f:h:s:e:r:vP:")) != -1) {
        switch (ch) {
        case 't':
            CONNECT_TIMEOUT = atoi(optarg);
            break;
        case 'w':
            wflags = 1; 
            html = optarg;
            break;
        case 'G':
            gflags = 1;
            break;
        case 'O':
            oflags = 1;
            break;
        case 'p':
            http_port = atoi(optarg);
            break;
        case 'f':
            fflags = 1;
            filename = optarg;
            break;
        case 'h':
            hflags = 1;
            server = optarg;
            break;
        case 's':
            start = inet_addr(optarg);
            break;
        case 'e':
            end = inet_addr(optarg);
            break;
        case 'r':
            rflags = 1;
            rulefile = optarg;
            break;
        case 'v':
            statuscode = 2;
            break;
        case 'P':
            MAX_PROCESSES = atoi(optarg);
            RULE_PROCESSES = atoi(optarg);
            IPRANGE_PROCESSES = atoi(optarg);
            break;
        default:
            usage();
        }
    }

    if (hflags) {
        if ((slash = strchr(server, '/')) != NULL)
            mflags = 1;
    }   

    if (start && end) {
        if (start > end) {
            fprintf(stdout, "start end address incorrect \n\n");
            exit(0);
        }
    }
    
    fprintf(stdout, "starting arirang %s    http://www.monkey.org/~pilot/arirang/ \n\n\n", VERSION);

    if (mflags) {
        *slash++ = '\0'; 
        cidr = inet_addr(server);
        bits = atoi(slash);

        if (bits > 0 && bits < 33) { 
            mask = 0xFFFFFFFF << (32 - bits);
#ifdef DEBUG
            printf("bits: %d,  mask: %lu\n", bits, mask);
#endif
            cidr = ntohl(cidr);
            start = htonl(cidr & mask);
            end = htonl(cidr | ~mask);
        }
        else {
            fprintf(stdout, "bits incorrect \n\n");
            exit(0);
        }
    }

    if (wflags) {
                if ((htmlreport = fopen(html, "w+")) == NULL) {
                        fprintf(stderr, "File %s write error\n", html);
                        exit(1);
                }
        fprintf(stdout, "arirang scan report file - %s \n\n", html);
        htmlhead(html);
    }


    if (gflags && hflags && mflags == 0)
        grabhead(server, http_port, 0);

    if (oflags && hflags && mflags == 0)
        grabhead(server, http_port, 1);

    if (fflags == 1 && rflags == 0 && mflags == 0) {

        if ((scanlist = fopen(filename, "rt")) == NULL) {
            fprintf(stderr, "File %s open error\n", filename);
            exit(0);
        }

/* -f with process */
#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
        signal(SIGALRM, timeout);
#endif
        pcount = 0;
        while (!feof(scanlist)) {
            fscanf(scanlist, "%s\n", list);

/* check domain or ip */
        fip = list;
        while (*fip != '\0') {
        if ((*fip >= 'a' && *fip <= 'z')  || (*fip >= 'A' && *fip <= 'Z'))
            fnum = 1;
         fip++;
        }

        if (fnum == 0) {
            fcount =  inet_addr(list);
            fcount = ntohl(fcount);
                                                                                          
            if ((fcount & 0xff) == 0 || (fcount & 0xff) == 255) 
                continue;
        }
        fnum = 0;

            if (pcount > MAX_PROCESSES)
                wait(NULL);

            switch (fork()) {
            case 0:
                alarm(2);
#ifdef DEBUG
            printf("%s \n", list);
#endif
            if (gflags)
                grabhead(list, http_port, 0);
            if (oflags)
                grabhead(list, http_port, 1);

                exit(0);
            case -1:
                perror("fork err");
                exit(-1);
            default:
                pcount++;
                break;
            }
        }
        fclose(scanlist);

        while (pcount--)
            wait(NULL);
    }

    if ((start && end && rflags == 0) || (mflags == 1 && rflags == 0)) {
#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
        signal(SIGALRM, timeout);
#endif
        pcount = 0;

        for (count = ntohl(start); count <= ntohl(end); count++) {
            if ((count & 0xff) == 0 || (count & 0xff) == 255)
                continue;
            if (pcount > MAX_PROCESSES)
                wait(NULL);

            switch (fork()) {
            case 0:
#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
                alarm(2);
#endif
                if (oflags)
                    grabheadW(count, http_port, 0, 1);
                else
                    grabheadW(count, http_port, 0, 0);
                
                exit(0);
            case -1:
                perror("fork");
                exit(-1);
            default:
                pcount++;
                break;
            }
        }
        while (pcount--)
            wait(NULL);
    }
    if (rflags && statuscode && hflags && mflags == 0) {
        if (RULE_PROCESSES == 0)
            scanrule(server, http_port, rulefile, statuscode);
        if (RULE_PROCESSES != 0)
            scanruleP(server, http_port, rulefile, statuscode, RULE_PROCESSES);
    }
/* -f with rule */
    if (rflags && statuscode && fflags && mflags == 0) {
        ftype = 1;
        if ((tmp_iprange = fopen(USED_IP_RANGE, "w")) == NULL) {
            fprintf(stderr, "File %s write error", USED_IP_RANGE);
            exit(1);
        }
        fclose(tmp_iprange);
        fprintf(stdout, "wait a please, arirang make a %s file \n\n\n", USED_IP_RANGE);

        if ((scanlist = fopen(filename, "rt")) == NULL) {
            fprintf(stderr, "File %s open error\n", filename);
            exit(0);
        }

#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
        signal(SIGALRM, timeout);
#endif
        pcount = 0;
        while (!feof(scanlist)) {
            fscanf(scanlist, "%s\n", list);

/* check domain or ip */
        fip = list;
        while (*fip != '\0') {
        if ((*fip >= 'a' && *fip <= 'z')  || (*fip >= 'A' && *fip <= 'Z'))
            fnum = 1;
        fip++;
        }

        if (fnum == 0) {
            fcount =  inet_addr(list);
            fcount = ntohl(fcount);

            if ((fcount & 0xff) == 0 || (fcount & 0xff) == 255)
                continue;
         }
        fnum = 0;

            if (pcount > MAX_PROCESSES)
                wait(NULL);

            switch (fork()) {
            case 0:
                alarm(2);

            if (gflags)
                grabhead(list, http_port, 0);
            if (oflags)
                grabhead(list, http_port, 1);

                exit(0);
            case -1:
                perror("fork err");
                exit(-1);
            default:
                pcount++;
                break;
            }
        }
        fclose(scanlist);

        while (pcount--)
            wait(NULL);

        ftype = 0;

        if ((scanlist = fopen(USED_IP_RANGE, "rt")) == NULL) {
            fprintf(stderr, "File %s open error\n", USED_IP_RANGE);
            exit(1);
        }
        while (!feof(scanlist)) {
            fscanf(scanlist, "%s\n", list);
            if (oflags)
                grabhead(list, http_port, 1);
            else
                grabhead(list, http_port, 0);

            scanruleP(list, http_port, rulefile, statuscode, IPRANGE_PROCESSES);
        }
        fclose(scanlist);
    }

    if ((rflags && start && end) || (rflags && mflags == 1)) {
        if ((tmp_iprange = fopen(USED_IP_RANGE, "w")) == NULL) {
            fprintf(stderr, "File %s write error", USED_IP_RANGE);
            exit(1);
        }
        fclose(tmp_iprange);
        fprintf(stdout, "wait a please, arirang make a %s file \n\n\n", USED_IP_RANGE);

#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
        signal(SIGALRM, timeout);
#endif
        pcount = 0;

        for (count = ntohl(start); count <= ntohl(end); count++) {
            if ((count & 0xff) == 0 || (count & 0xff) == 255)
                continue;
            if (pcount > IPRANGE_PROCESSES)
                wait(NULL);

            switch (fork()) {
            case 0:
#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
                alarm(2);
#endif
                if (oflags)
                    grabheadW(count, http_port, 1, 1);
                else
                    grabheadW(count, http_port, 1, 0);
                exit(0);
            case -1:
                perror("fork");
                exit(-1);
            default:
                pcount++;
                break;
            }
        }
        while (pcount--)
            wait(NULL);

        if ((scanlist = fopen(USED_IP_RANGE, "rt")) == NULL) {
            fprintf(stderr, "File %s open error\n", USED_IP_RANGE);
            exit(1);
        }
        while (!feof(scanlist)) {
            fscanf(scanlist, "%s\n", list);
            if (oflags)
                grabhead(list, http_port, 1);
            else
                grabhead(list, http_port, 0);

            scanruleP(list, http_port, rulefile, statuscode, IPRANGE_PROCESSES);
        }
        fclose(scanlist);

    }

    gettimeofday(&tv2, 0);

    if (tv2.tv_usec <= tv1.tv_usec) { 
        tv2.tv_sec--;
        tv2.tv_usec = tv2.tv_usec + 1000000;
    } 

    sec = tv2.tv_sec - tv1.tv_sec;
    usec = (tv2.tv_usec - tv1.tv_usec) / 1000; 
    fprintf(stdout, "\narirang scan time: %ld.%ld seconds\n\n", sec, usec);

    return 0;
}
