/*
 * Copyright 2001-2002, 2010 pilot <pilot@monkey.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>

#include "arirang.h"

extern char *html;
extern FILE *htmlreport;
extern int wflags;
extern int ftype;
extern short int CONNECT_TIMEOUT;

#if !(defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__))
int sock;

void timegrab(int sig)
{
    close(sock);
}
#endif

int
recvtimeout(int s, char *buf, int len, int timeout)
{
    fd_set  fds;
    int n;
    struct timeval  tv;

    FD_ZERO(&fds);
    FD_SET(s, &fds);
    tv.tv_sec = timeout;
    tv.tv_usec = 0;

    n = select(s + 1, &fds, NULL, NULL, &tv);
    if (n == 0)
        return -2;
    if (n == -1)
        return -1;

    return recv(s, buf, len, 0);
}

void
grabhead(char *scanhost, unsigned int port, short int option)
{
    int sock, nb;
    struct sockaddr_in http;
    struct hostent *he;

    char osfinger[32];
    char headBuff[512];

    char *ptr, *ptr2, *buffer_pointer;
    FILE *tmp_iprange;

    if (wflags) {
        if ((htmlreport = fopen(html, "a+")) == NULL) {
            fprintf(stderr, "File %s write error\n", html);
            exit(1);
            }
    }

    if ((he = gethostbyname(scanhost)) == NULL) {
        perror("gethostbyname");
        /* close(sock);  */
        exit(0);
    }
    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        perror("socket");
        close(sock);
        exit(0);
    }
    http.sin_family = AF_INET;
    http.sin_port = htons(port);
    http.sin_addr = *((struct in_addr *) he->h_addr);
    bzero(&(http.sin_zero), 8);

    if (connect(sock, (struct sockaddr *) & http, sizeof(struct sockaddr)) == -1) {
        close(sock);
        exit(0);
    }

    if (option == 1)
        snprintf(osfinger, sizeof(osfinger), "%s\n\n", SEND_OPTION);
    else
        snprintf(osfinger, sizeof(osfinger), "%s\n\n", SEND_URI);

    send(sock, osfinger, strlen(osfinger), 0);
    nb = recvtimeout(sock, headBuff, sizeof(headBuff), CONNECT_TIMEOUT);

    if (nb == -1 || nb == -2) {
        if (wflags)
            fprintf(htmlreport, "<font size=\"2\">%s - maybe router or unknown network device<br></font>", scanhost);
        else
            fprintf(stdout, "%s - maybe router or unknown network device\n", scanhost);
        close(sock);
        exit(0);
    }
    if (strstr(headBuff, "3Com") != NULL) {
        close(sock);
        exit(0);
    }

/* f + r */
    if (ftype == 1) {
        if ((tmp_iprange = fopen(USED_IP_RANGE, "a")) == NULL) {
            fprintf(stderr, "File %s write error", USED_IP_RANGE);
            exit(1);
        } else {
            fprintf(tmp_iprange, "%s\n", scanhost);
            fclose(tmp_iprange);
        }
    } else {
            if (option == 1)
                ptr = strstr(headBuff, "Allow:");
            else
                ptr = strstr(headBuff, "Server:");
       }

    if (ptr == NULL) {
        if (wflags)
            fprintf(htmlreport, "<font size=\"2\">%s - could not get information in header</font><br><br>", scanhost);
        else
            fprintf(stdout, "%s - could not get information in header\n", scanhost);
    } else {
        ptr2 = strstr(ptr, "\n");

        *ptr2 = '\0';
        buffer_pointer = strdup(ptr);
        *ptr2 = '\n';

        if (wflags)
            fprintf(htmlreport, "<br><br><font size=\"2\"><b><a href=\"http://%s\">%s</a> - %s<br></b></font>", 
                scanhost, scanhost, buffer_pointer);
        else
            fprintf(stdout, "%s - %s\n", scanhost, buffer_pointer);
        free(buffer_pointer);
    }

    if (wflags)
        fclose(htmlreport);

    close(sock);

}



void
grabheadW(unsigned long host, unsigned int port, unsigned short int type, short int option)
{
#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
    int sock;
#endif
    int nb;        
    struct sockaddr_in http;


    char osfinger[32];
    char headBuff[512];

    char *ptr, *ptr2, *buffer_pointer;
    FILE *tmp_iprange;

#if !(defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__))
    signal(SIGALRM,timegrab);
    alarm(2);
#endif

    sock = socket(AF_INET, SOCK_STREAM, 0);
    http.sin_family = AF_INET;
    http.sin_port = htons(port);
    http.sin_addr.s_addr = htonl(host);

#ifdef DEBUG
    printf("Debug %s\n", inet_ntoa(http.sin_addr)); 
#endif

    if (wflags) {
        if ((htmlreport = fopen(html, "a+")) == NULL) {
            fprintf(stderr, "File %s write error\n", html);
            exit(1);
        }
    }

    if (connect(sock, (struct sockaddr *) & http, sizeof(struct sockaddr)) == -1) {
        close(sock);
        exit(0);
    }

    if (option == 1)
        snprintf(osfinger, sizeof(osfinger), "%s\n\n", SEND_OPTION);
    else
        snprintf(osfinger, sizeof(osfinger), "%s\n\n", SEND_URI);

    send(sock, osfinger, strlen(osfinger), 0);
    nb = recvtimeout(sock, headBuff, sizeof(headBuff), CONNECT_TIMEOUT);

    if (nb == -1 || nb == -2) { 
#if !(defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__))
    alarm(0);
#endif
    close(sock);
    exit(0);
    }

    if (strstr(headBuff, "Server Error") != NULL) {
#if !(defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__))
    alarm(0);
#endif
    close(sock);
    exit(0);
    }

    if (type == 1) {
        if ((tmp_iprange = fopen(USED_IP_RANGE, "a")) == NULL) {
            fprintf(stderr, "File %s write error", USED_IP_RANGE);
            exit(1);
        } else {
            fprintf(tmp_iprange, "%s\n", inet_ntoa(http.sin_addr));
            fclose(tmp_iprange);
        }
    } else {
            if (option == 1)
                ptr = strstr(headBuff, "Allow:");
            else
                ptr = strstr(headBuff, "Server:");

    if (ptr == NULL) {
        if (wflags)
            fprintf(htmlreport, "<br><font size=\"2\">%s - could not get information in header</font><br>", inet_ntoa(http.sin_addr));
        else
            fprintf(stdout, "%s - could not get information in header\n", inet_ntoa(http.sin_addr));
    } else {
        ptr2 = strstr(ptr, "\n");

        *ptr2 = '\0';
        buffer_pointer = strdup(ptr);
        *ptr2 = '\n';

        if (wflags)
            fprintf(htmlreport, "<br><font size=\"2\"><b><a href=\"http://%s\">%s</a> - %s</b><br>", 
                inet_ntoa(http.sin_addr), inet_ntoa(http.sin_addr), buffer_pointer);
            else
                fprintf(stdout, "%s - %s\n", inet_ntoa(http.sin_addr), buffer_pointer);

            free(buffer_pointer);
        }
    }

    if (wflags)
        fclose(htmlreport);

    close(sock);
#if !(defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__))
    exit(0);
#endif
}
